!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                                       !
!                     Surface Fluxes I/O Component of ELCIRC                            !
!                           Version 8 (January  5, 2006)                                !
!                                                                                       !
!                 Center for Coastal and Land-Margin Research                           !
!             Department of Environmental Science and Engineering                       !
!                   OGI School of Science and Engineering,                              !
!                     Oregon Health & Science University                                !
!                       Beaverton, Oregon 97006, USA                                    !
!                                                                                       !
!                   Scientific direction: Antonio Baptista                              !
!                   Code development: Mike A. Zulauf                  			!
!                                                                                       !
!               Copyright 1999-2006 Oregon Health and Science University                !
!                              All Rights Reserved                                      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  
!
!-----------------------------------------------------------------------
! Note: the following global variables (from module global) are used in
!       this code.  This list does not include variables passed in as
!       arguments. . .
!
!       mnp
!       np
!       xlon
!       ylat
!-----------------------------------------------------------------------
!
! Note: this file contains two primary subroutines, get_wind and
!       get_rad.  Their calling convention is defined below.
!
!
!     call get_wind (time, u_air_node, v_air_node, p_air_node, &
!    &               t_air_node, q_air_node)
!
!     scalar
!       time:       model time in seconds
!
!     vectors defined at nodes
!       u_air_node: east/west component of near-surface wind at a
!                   height of 10m, m/s
!       v_air_node: north/south component of near-surface wind at a
!                   height of 10m, m/s
!       p_air_node: surface atmospheric pressure, Pa
!       t_air_node: near-surface air temperature at a height of 2m,
!                   deg K
!       q_air_node: near-surface specific humidity at a height of 2m,
!                   kg/kg
!
!
!     call get_rad (time, shortwave_d, longwave_d)
!
!     scalar
!       time:       model time in seconds
!
!     vectors defined at nodes
!       shortwave_d: downwelling shortwave (solar) radiation at the
!                    surface, W/m^2
!       longwave_d: downwelling longwave (IR) radiation at the
!                    surface, W/m^2
!-----------------------------------------------------------------------
!
! Two versions are given here.  The first (with USE_HDF preprocessing
! conditional _not_ defined) is supplied here primarily as guidance
! for users to write their own versions.  One could specify time- and
! space-varying values, or constant values, etc.
!
! The second version (with the USE_HDF preprocessing conditional
! defined) uses the ELCIRC/SELFE HDF I/O component to specify the time-
! and space-varying forcing quantities.
!
! The second version needs to be linked with the HDF4 libraries.
!
!-----------------------------------------------------------------------
#ifndef USE_HDF /* USE_HDF not defined */
!
! This subroutine returns the time/space-varying near surface wind
! components, air pressure, air temperature, and specific humidity.
!
      subroutine get_wind (time, u_air_node, v_air_node, p_air_node, &
     &                     t_air_node, q_air_node)

!       implicit none
        use global, only : mnp, np, xlon, ylat
        implicit real*8(a-h,o-z),integer(i-n)

! define some new names for things in header file
        integer max_nodes
        parameter (max_nodes = mnp)

! input/output variables
        real*8 u_air_node(max_nodes), v_air_node(max_nodes)
        real*8 t_air_node(max_nodes), q_air_node(max_nodes)
        real*8 p_air_node(max_nodes), time

! local variables
        integer num_nodes, i_node

! define the local variables num_nodes
        num_nodes = np

        open(39,file='fort.39')
        rewind(39)
        write(39,*)
        write(39,*) 'enter get_wind'

        write(39,*) 'specifying surface properties'
        do i_node = 1, num_nodes
          u_air_node(i_node) = 0.0d0
          v_air_node(i_node) = 0.0d0
          t_air_node(i_node) = 280.0d0
          q_air_node(i_node) = 8.0d-3
          p_air_node(i_node) = 1.01d5
        enddo

        close(39)

      return
      end
!-----------------------------------------------------------------------
!
! This subroutine returns the time/space-varying downwelling
! shortwave (solar) and longwave (IR) radiative fluxes.
!
! NOTE: solar radiation has been reduced to account for surface albedo.
!
      subroutine get_rad (time, shortwave_d, longwave_d)

!       implicit none
        use global, only : mnp, np
        implicit real*8(a-h,o-z),integer(i-n)

! define some new names for things in header file
        integer max_nodes
        parameter (max_nodes = mnp)

! input/output variables
        real*8 time, longwave_d(max_nodes), shortwave_d(max_nodes)

! local variables
        integer num_nodes, i_node

! define the local variables num_nodes
        num_nodes = np

        open(40,file='fort.40')
        rewind(40)
        write(40,*)
        write(40,*) 'enter get_rad'

        write(40,*) 'specifying shortwave and longwave'
        do i_node = 1, num_nodes
          shortwave_d(i_node) = 100.0d0
          longwave_d(i_node) = 350.0d0
        enddo

        close(40)

      return
      end
!-----------------------------------------------------------------------
!
#else           /* USE_HDF is defined */
!
! This subroutine returns the time/space-varying near surface wind
! components, air pressure, air temperature, and specific humidity.
!
      subroutine get_wind (time, u_air_node, v_air_node, p_air_node, &
     &                     t_air_node, q_air_node)

!       implicit none
        use global, only : mnp, np, xlon, ylat
        implicit real*8(a-h,o-z),integer(i-n)

! define some new names for things in header file
        integer max_nodes
        parameter (max_nodes = mnp)

! input/output variables
        real*8 u_air_node(max_nodes), v_air_node(max_nodes)
        real*8 t_air_node(max_nodes), q_air_node(max_nodes)
        real*8 p_air_node(max_nodes), time

! local variables
        integer max_ni, max_nj, max_times, max_files
        integer max_elems_in, max_nodes_in
        parameter (max_ni = 1152)
        parameter (max_nj = 576)
        parameter (max_elems_in = (max_ni-1) * (max_nj-1) * 2)
        parameter (max_nodes_in = max_ni * max_nj)
        parameter (max_times = 1000)
        parameter (max_files = 999)
        integer num_nodes
        integer in_elem_to_out_node_1(max_nodes)
        integer in_elem_to_out_node_2(max_nodes)
        integer ni_1, nj_1, num_times_1, num_nodes_in_1, num_elems_in_1
        integer node_i_1(max_nodes_in), node_j_1(max_nodes_in)
        integer node_num_in_1(max_nodes_in)
        integer elem_nodes_in_1(max_elems_in,3)
        integer ni_2, nj_2, num_times_2, num_nodes_in_2, num_elems_in_2
        integer node_i_2(max_nodes_in), node_j_2(max_nodes_in)
        integer node_num_in_2(max_nodes_in)
        integer elem_nodes_in_2(max_elems_in,3)
        integer num_files_1, num_files_2
        integer max_rank, rank
        parameter (max_rank = 3)
        integer dim_sizes(max_rank)
        real*8 weight_wind_node_1(max_nodes,3)
        real*8 weight_wind_node_2(max_nodes,3)
        real*8 x_in_1(max_ni,max_nj), y_in_1(max_ni,max_nj)
        real*8 x_in_2(max_ni,max_nj), y_in_2(max_ni,max_nj)
        real*8 start_day_1
        real*8 relative_weight_1, relative_weight_2
        real*8 max_window_1, max_window_2
        parameter (relative_weight_1 = 1.0)
#ifndef MM5     /* MM5 not defined  */
        parameter (relative_weight_2 = 2.0)
#else           /* MM5 is defined   */
        parameter (relative_weight_2 = 1000.0)
#endif          /* end of MM5 block */
        parameter (max_window_1 = 24.0)
        parameter (max_window_2 = 2.0)
        real*8 frac_day, secs_per_day, utc_start
        parameter (secs_per_day = 86400.0)
        parameter (utc_start = 8.0)
        real*4 temp_arr_2(max_ni,max_nj), temp_arr_3(max_ni,max_nj)
        real*4 temp_arr_4(max_ni,max_nj), temp_arr_5(max_ni,max_nj)
        real*8 temp_arr_1(max_ni,max_nj), temp_arr_7(max_ni,max_nj)
        real*8 temp_arr_6(max_elems_in)
        real*8 temp_arr_8(max_nodes), temp_arr_9(max_nodes)
        integer temp_arr_i(max_ni,max_nj)
        real*4 temp_sca
        real*4 wind_times_1(max_times), wind_times_2(max_times)
        character wind_set_1*50, wind_set_2*50, start_day_file*50
        character wind_time_files_1(max_times)*50
        character wind_time_files_2(max_times)*50
        parameter (wind_set_1 = 'hdf/wind_file_1')
        parameter (wind_set_2 = 'hdf/wind_file_2')
        parameter (start_day_file = 'hdf/start_day.txt')
        logical first_call, have_wind_2, have_start_day_file
        data first_call/.true./

! retain the values of some local variables between calls
        save first_call, start_day_1, &
     &    in_elem_to_out_node_1, weight_wind_node_1, &
     &    in_elem_to_out_node_2, weight_wind_node_2, &
     &    have_wind_2, &
     &    wind_times_1, num_times_1, num_files_1, ni_1, nj_1, &
     &    num_nodes_in_1, num_elems_in_1, node_i_1, node_j_1, &
     &    node_num_in_1, elem_nodes_in_1, &
     &    wind_times_2, num_times_2, num_files_2, ni_2, nj_2, &
     &    num_nodes_in_2, num_elems_in_2, node_i_2, node_j_2, &
     &    node_num_in_2, elem_nodes_in_2, &
     &    wind_time_files_1, wind_time_files_2

        open(39,file='fort.39')
        rewind(39)
        write(39,*)
        write(39,*) 'enter get_wind'
        write(39,*) 'first_call = ', first_call

! define the local variable num_nodes
        num_nodes = np

! if this is the first call to this routine then get some things ready
        if (first_call) then

! check to see if start_day_file exists
          call file_exst (start_day_file, have_start_day_file, .false.)

! if the start day file does exist, get start_day from it, otherwise
! use the first start_day in wind_set_1
          if (have_start_day_file) then
            open (unit=77, file=start_day_file, status='old')
            read(77,*) temp_sca
            close (unit=77)
            start_day_1 = temp_sca
          else
            wind_time_files_1(1) = 'hdf/wind_file_1.001.hdf'
            call read_scalar(wind_time_files_1(1), temp_sca, &
     &                       'start_day           ', 0.0)
            start_day_1 = temp_sca
          endif

! check to see if _any_ wind_file_2 exists (use first possible name)
          wind_time_files_2(1) = 'hdf/wind_file_2.001.hdf'
          call file_exst (wind_time_files_2(1), have_wind_2, .false.)
          if (.not. have_wind_2) then
            write(39,*)
            write(39,*) wind_time_files_2(1), ' not exist. . .'
          endif

! get the times of the data available in wind_set_1
          call get_times(wind_times_1, wind_set_1, &
     &                   'u                   ', &
     &                   wind_time_files_1, num_times_1, &
     &                   num_files_1, max_times, max_files)

! get the dimensions of the datasets in wind_set_1 (use first dataset)
          call get_dims(wind_time_files_1(1), 'u                   ', &
     &                  wind_times_1(1), rank, dim_sizes)
          ni_1 = dim_sizes(1)
          nj_1 = dim_sizes(2)

! check the dimensions of wind_set_1, to ensure they don't exceed the
! maximums
          if (ni_1 .gt. max_ni .or. nj_1 .gt. max_nj) then
            write(11,*)
            write(11,*) 'wind_file_1: max dimensions exceeded!'
            stop
          endif

! calculate the total number of nodes and elements for wind_set_1
          num_nodes_in_1 = ni_1 * nj_1
          num_elems_in_1 = (ni_1-1) * (nj_1-1) * 2

! check the elems/nodes of wind_set_1, to ensure they don't exceed the
! maximums
          if (num_elems_in_1 .gt. max_elems_in .or. &
     &        num_nodes_in_1 .gt. max_nodes_in) then
            write(11,*)
            write(11,*) 'wind_file_1: max elems/nodes exceeded!'
            stop
          endif

! create list of all nodes for wind_set_1
          call list_nodes (node_i_1, node_j_1, node_num_in_1, &
     &                     num_nodes_in_1, ni_1, nj_1)

! now create the list of all the elements (and the nodes defining them)
! for wind_set_1
          call list_elems (elem_nodes_in_1, node_num_in_1, &
     &                     ni_1, nj_1, num_elems_in_1)

! do the same as above for wind_set_2 (if it exists)
          if (have_wind_2) then

! get the times of the data available in wind_set_2
            call get_times(wind_times_2, wind_set_2, &
     &                     'u                   ', &
     &                     wind_time_files_2, num_times_2, &
     &                     num_files_2, max_times, max_files)

! get the dimensions of the datasets in wind_set_2 (use first dataset)
            call get_dims(wind_time_files_2(1), 'u                   ', &
     &                    wind_times_2(1), rank, dim_sizes)
            ni_2 = dim_sizes(1)
            nj_2 = dim_sizes(2)

! check the dimensions of wind_set_2, to ensure they don't exceed the
! maximums
            if (ni_2 .gt. max_ni .or. nj_2 .gt. max_nj) then
              write(11,*)
              write(11,*) 'wind_file_2: max dimensions exceeded!'
              stop
            endif

! calculate the total number of nodes and elements for wind_set_2
            num_nodes_in_2 = ni_2 * nj_2
            num_elems_in_2 = (ni_2-1) * (nj_2-1) * 2

! check the elems/nodes of wind_set_2, to ensure they don't exceed the
! maximums
            if (num_elems_in_2 .gt. max_elems_in .or. &
     &          num_nodes_in_2 .gt. max_nodes_in) then
              write(11,*)
              write(11,*) 'wind_file_2: max elems/nodes exceeded!'
              stop
            endif

! create list of all nodes for wind_set_2
            call list_nodes (node_i_2, node_j_2, node_num_in_2, &
     &                       num_nodes_in_2, ni_2, nj_2)

! now create the list of all the elements (and the nodes defining them)
! for wind_set_2
            call list_elems (elem_nodes_in_2, node_num_in_2, &
     &                       ni_2, nj_2, num_elems_in_2)

          endif ! end of have_wind_2 block

! get the x and y values for wind_set_1 (full size real*8 arrays)
          call get_xy (wind_time_files_1(1), temp_arr_2, temp_arr_3, &
     &                 x_in_1, y_in_1, &
     &                 temp_arr_1, temp_arr_7, temp_arr_i, &
     &                 ni_1, nj_1, max_ni, max_nj)

! calculate the weightings from wind_set_1 to elcirc nodes
! (this is slow)
          write(16,*)
          write(16,*) &
     &      'begin calculating grid weightings for wind_file_1'
          call get_weight (x_in_1, y_in_1, xlon, ylat, &
     &                     elem_nodes_in_1, node_i_1, node_j_1, &
     &                     max_ni, max_nj, &
     &                     num_elems_in_1, num_nodes_in_1, &
     &                     num_nodes, &
     &                     max_nodes, &
     &                     in_elem_to_out_node_1, &
     &                     temp_arr_6, weight_wind_node_1)
          write(16,*) &
     &      'done calculating grid weightings for wind_file_1'

! do the same but for wind_set_2 (if it exists)
          if (have_wind_2) then

! get the x and y values for wind_set_2 (full size real*8 arrays)
          call get_xy (wind_time_files_2(1), temp_arr_2, temp_arr_3, &
     &                 x_in_2, y_in_2, &
     &                 temp_arr_1, temp_arr_7, temp_arr_i, &
     &                 ni_2, nj_2, max_ni, max_nj)

! calculate the weightings from wind_set_2 to elcirc nodes
! (this is slow)
            write(16,*)
            write(16,*) &
     &        'begin calculating grid weightings for wind_file_2'
            call get_weight (x_in_2, y_in_2, xlon, ylat, &
     &                       elem_nodes_in_2, node_i_2, node_j_2, &
     &                       max_ni, max_nj, &
     &                       num_elems_in_2, num_nodes_in_2, &
     &                       num_nodes, &
     &                       max_nodes, &
     &                       in_elem_to_out_node_2, &
     &                       temp_arr_6, weight_wind_node_2)
            write(16,*) &
     &        'done calculating grid weightings for wind_file_2'

          endif

! output starting date and time
          write(16,*)
          write(16,*) 'wind file starting Julian date: ', start_day_1
          write(16,*) 'wind file assumed UTC starting time: ', utc_start

        endif ! (end of first_call block)

! define frac_day - the fractional Julian date
! include offset for starting time in UTC (in hours)
        frac_day = start_day_1 + time/secs_per_day + utc_start/24.0

! output info to debug file
        write(39,*) 'num_nodes = ', num_nodes
        write(39,*) 'num_files_1 = ', num_files_1
        write(39,*) 'num_times_1 = ', num_times_1
        if (have_wind_2) then
          write(39,*) 'num_files_2 = ', num_files_2
          write(39,*) 'num_times_2 = ', num_times_2
        endif
        write(39,*)
        write(39,*) 'start_day = ', start_day_1
        write(39,*) 'time      = ', time
        write(39,*) 'frac_day  = ', frac_day

! now retrieve data from the wind files

        call combine_set (real(frac_day), 'u                   ', &
     &                    wind_set_1, wind_set_2, &
     &                    temp_arr_2, temp_arr_3, &
     &                    temp_arr_4, temp_arr_5, temp_arr_1, &
     &                    wind_times_1, num_times_1, ni_1, nj_1, &
     &                    wind_times_2, num_times_2, ni_2, nj_2, &
     &                    num_files_1, num_files_2, max_files, &
     &                    temp_arr_8, temp_arr_9, u_air_node, &
     &                    in_elem_to_out_node_1, weight_wind_node_1, &
     &                    in_elem_to_out_node_2, weight_wind_node_2, &
     &                    elem_nodes_in_1, node_i_1, node_j_1, &
     &                    elem_nodes_in_2, node_i_2, node_j_2, &
     &                    max_ni, max_nj, max_nodes_in, &
     &                    num_nodes, max_nodes, &
     &                    num_nodes_in_1, num_elems_in_1, &
     &                    num_nodes_in_2, num_elems_in_2, &
     &                    max_elems_in, have_wind_2, &
     &                    relative_weight_1, relative_weight_2, &
     &                    max_window_1, max_window_2, &
     &                    wind_time_files_1, wind_time_files_2)

        call combine_set (real(frac_day), 'v                   ', &
     &                    wind_set_1, wind_set_2, &
     &                    temp_arr_2, temp_arr_3, &
     &                    temp_arr_4, temp_arr_5, temp_arr_1, &
     &                    wind_times_1, num_times_1, ni_1, nj_1, &
     &                    wind_times_2, num_times_2, ni_2, nj_2, &
     &                    num_files_1, num_files_2, max_files, &
     &                    temp_arr_8, temp_arr_9, v_air_node, &
     &                    in_elem_to_out_node_1, weight_wind_node_1, &
     &                    in_elem_to_out_node_2, weight_wind_node_2, &
     &                    elem_nodes_in_1, node_i_1, node_j_1, &
     &                    elem_nodes_in_2, node_i_2, node_j_2, &
     &                    max_ni, max_nj, max_nodes_in, &
     &                    num_nodes, max_nodes, &
     &                    num_nodes_in_1, num_elems_in_1, &
     &                    num_nodes_in_2, num_elems_in_2, &
     &                    max_elems_in, have_wind_2, &
     &                    relative_weight_1, relative_weight_2, &
     &                    max_window_1, max_window_2, &
     &                    wind_time_files_1, wind_time_files_2)

        call combine_set (real(frac_day), 'p_msl               ', &
     &                    wind_set_1, wind_set_2, &
     &                    temp_arr_2, temp_arr_3, &
     &                    temp_arr_4, temp_arr_5, temp_arr_1, &
     &                    wind_times_1, num_times_1, ni_1, nj_1, &
     &                    wind_times_2, num_times_2, ni_2, nj_2, &
     &                    num_files_1, num_files_2, max_files, &
     &                    temp_arr_8, temp_arr_9, p_air_node, &
     &                    in_elem_to_out_node_1, weight_wind_node_1, &
     &                    in_elem_to_out_node_2, weight_wind_node_2, &
     &                    elem_nodes_in_1, node_i_1, node_j_1, &
     &                    elem_nodes_in_2, node_i_2, node_j_2, &
     &                    max_ni, max_nj, max_nodes_in, &
     &                    num_nodes, max_nodes, &
     &                    num_nodes_in_1, num_elems_in_1, &
     &                    num_nodes_in_2, num_elems_in_2, &
#ifndef MM5     /* MM5 not defined  */
     &                    max_elems_in, have_wind_2, &
#else           /* MM5 is defined   */
     &                    max_elems_in, .false., &
#endif          /* end of MM5 block */
     &                    relative_weight_1, relative_weight_2, &
     &                    max_window_1, max_window_2, &
     &                    wind_time_files_1, wind_time_files_2)

        call combine_set (real(frac_day), 'T                   ', &
     &                    wind_set_1, wind_set_2, &
     &                    temp_arr_2, temp_arr_3, &
     &                    temp_arr_4, temp_arr_5, temp_arr_1, &
     &                    wind_times_1, num_times_1, ni_1, nj_1, &
     &                    wind_times_2, num_times_2, ni_2, nj_2, &
     &                    num_files_1, num_files_2, max_files, &
     &                    temp_arr_8, temp_arr_9, t_air_node, &
     &                    in_elem_to_out_node_1, weight_wind_node_1, &
     &                    in_elem_to_out_node_2, weight_wind_node_2, &
     &                    elem_nodes_in_1, node_i_1, node_j_1, &
     &                    elem_nodes_in_2, node_i_2, node_j_2, &
     &                    max_ni, max_nj, max_nodes_in, &
     &                    num_nodes, max_nodes, &
     &                    num_nodes_in_1, num_elems_in_1, &
     &                    num_nodes_in_2, num_elems_in_2, &
     &                    max_elems_in, have_wind_2, &
     &                    relative_weight_1, relative_weight_2, &
     &                    max_window_1, max_window_2, &
     &                    wind_time_files_1, wind_time_files_2)

        call combine_set (real(frac_day), 'qv                  ', &
     &                    wind_set_1, wind_set_2, &
     &                    temp_arr_2, temp_arr_3, &
     &                    temp_arr_4, temp_arr_5, temp_arr_1, &
     &                    wind_times_1, num_times_1, ni_1, nj_1, &
     &                    wind_times_2, num_times_2, ni_2, nj_2, &
     &                    num_files_1, num_files_2, max_files, &
     &                    temp_arr_8, temp_arr_9, q_air_node, &
     &                    in_elem_to_out_node_1, weight_wind_node_1, &
     &                    in_elem_to_out_node_2, weight_wind_node_2, &
     &                    elem_nodes_in_1, node_i_1, node_j_1, &
     &                    elem_nodes_in_2, node_i_2, node_j_2, &
     &                    max_ni, max_nj, max_nodes_in, &
     &                    num_nodes, max_nodes, &
     &                    num_nodes_in_1, num_elems_in_1, &
     &                    num_nodes_in_2, num_elems_in_2, &
     &                    max_elems_in, have_wind_2, &
     &                    relative_weight_1, relative_weight_2, &
     &                    max_window_1, max_window_2, &
     &                    wind_time_files_1, wind_time_files_2)

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

        close(39)

      return
      end
!-----------------------------------------------------------------------
!
! This subroutine returns the time/space-varying downwelling
! shortwave (solar) and longwave (IR) radiative fluxes.
!
! NOTE: solar radiation has been reduced to account for surface albedo.
!
      subroutine get_rad (time, shortwave_d, longwave_d)

!       implicit none
        use global, only : mnp, np, xlon, ylat
        implicit real*8(a-h,o-z),integer(i-n)

! define some new names for things in header file
        integer max_nodes
        parameter (max_nodes = mnp)

! input/output variables
        real*8 time, longwave_d(max_nodes), shortwave_d(max_nodes)

! local variables
        integer num_nodes, i_node
        real*4 temp_sca, window
        real*8 weight_rad_node(max_nodes,3)
        real*8 start_day, frac_day, secs_per_day, utc_start
        real*8 max_window
        parameter (max_window = 6.0)
        parameter (secs_per_day = 86400.0)
        parameter (utc_start = 8.0)
        integer in_elem_to_out_node(max_nodes)
        character flux_set*50, start_day_file*50
        parameter (flux_set = 'hdf/flux_file_1')
        parameter (start_day_file = 'hdf/start_day.txt')
        logical first_call, time_exst, have_start_day_file
        data first_call/.true./
        integer max_ni, max_nj, ni, nj, max_times, num_times
        integer num_files, max_files
        integer num_nodes_in, num_elems_in
        integer max_elems_in, max_nodes_in
        parameter (max_ni = 1152)
        parameter (max_nj = 576)
        parameter (max_elems_in = (max_ni-1) * (max_nj-1) * 2)
        parameter (max_nodes_in = max_ni * max_nj)
        parameter (max_times = 1000)
        parameter (max_files = 999)
        real*4 temp_arr_2(max_ni,max_nj), temp_arr_3(max_ni,max_nj)
        real*4 temp_arr_4(max_ni,max_nj), temp_arr_5(max_ni,max_nj)
        real*8 temp_arr_1(max_ni,max_nj), temp_arr_7(max_ni,max_nj)
        real*8 temp_arr_6(max_elems_in)
        real*8 temp_arr_8(max_nodes)
        integer temp_arr_i(max_ni,max_nj)
        real*4 flux_times(max_times)
        real*8 shortwave_in(max_ni,max_nj), longwave_in(max_ni,max_nj)
        real*8 x_in(max_ni,max_nj), y_in(max_ni,max_nj)
        character flux_time_files(max_times)*50
        integer node_i(max_nodes_in), node_j(max_nodes_in)
        integer max_rank, rank
        integer node_num_in(max_nodes_in), elem_nodes_in(max_elems_in,3)
        parameter (max_rank = 3)
        integer dim_sizes(max_rank)
        integer printit

! retain the values of some local variables between calls
        save first_call, start_day, in_elem_to_out_node, &
     &       weight_rad_node, &
     &       flux_times, num_times, num_files, ni, nj, &
     &       num_nodes_in, num_elems_in, node_i, node_j, node_num_in, &
     &       elem_nodes_in, flux_time_files

        printit = 1000

        open(40,file='fort.40')
        rewind(40)
        write(40,*)
        write(40,*) 'enter get_rad'
        write(40,*) 'first_call = ', first_call

! define the local variable num_nodes
        num_nodes = np

! if this is the first call to this routine then get some things ready
        if (first_call) then

! check to see if start_day_file exists
          call file_exst (start_day_file, have_start_day_file, .false.)

! if the start day file does exist, get start_day from it, otherwise
! use the first start_day in flux_set
          if (have_start_day_file) then
            open (unit=77, file=start_day_file, status='old')
            read(77,*) temp_sca
            close (unit=77)
            start_day = temp_sca
          else
            flux_time_files(1) = 'hdf/flux_file_1.001.hdf'
            call read_scalar(flux_time_files(1), temp_sca, &
     &                       'start_day           ', 0.0)
            start_day = temp_sca
          endif

! get the times of the fluxes available in the flux_set
          call get_times(flux_times, flux_set, &
     &                   'shortwave_flux      ', &
     &                   flux_time_files, num_times, &
     &                   num_files, max_times, max_files)

! get the dimensions of the flux datasets (use first dataset)
          call get_dims(flux_time_files(1), 'shortwave_flux      ', &
     &                  flux_times(1), rank, dim_sizes)
          ni = dim_sizes(1)
          nj = dim_sizes(2)

! check the dimensions of the flux datasets, to ensure they don't exceed
! the maximums
          if (ni .gt. max_ni .or. nj .gt. max_nj) then
            write(11,*)
            write(11,*) 'Radiative flux data: max dimensions exceeded!'
            stop
          endif

! calculate the total number of nodes and elements for the radiative
! fluxes input grid
          num_nodes_in = ni * nj
          num_elems_in = (ni-1) * (nj-1) * 2

! check the elems/nodes of the input dataset, to ensure they don't
! exceed the maximums
          if (num_elems_in .gt. max_elems_in .or. &
     &        num_nodes_in .gt. max_nodes_in) then
            write(11,*)
            write(11,*) 'Radiative flux data: max elems/nodes exceeded!'
            stop
          endif

! create list of all nodes for input grid
          call list_nodes (node_i, node_j, node_num_in, &
     &                     num_nodes_in, ni, nj)

! now create the list of all the elements (and the nodes defining them)
          call list_elems (elem_nodes_in, node_num_in, &
     &                     ni, nj, num_elems_in)

! get the x and y values for the input grid (full size real*8 arrays)
          call get_xy (flux_time_files(1), temp_arr_4, temp_arr_5, &
     &                 x_in, y_in, &
     &                 temp_arr_1, temp_arr_7, temp_arr_i, &
     &                 ni, nj, max_ni, max_nj)

! calculate the weightings from rad grid to elcirc nodes (this is slow)
          write(16,*)
          write(16,*) 'calculating grid weightings for rad fluxes'
          call get_weight (x_in, y_in, xlon, ylat, &
     &                     elem_nodes_in, node_i, node_j, &
     &                     max_ni, max_nj, &
     &                     num_elems_in, num_nodes_in, &
     &                     num_nodes, &
     &                     max_nodes, &
     &                     in_elem_to_out_node, &
     &                     temp_arr_6, weight_rad_node)

! output starting date and time
          write(16,*)
          write(16,*) 'rad fluxes file starting Julian date: ', &
     &                start_day
          write(16,*) 'rad fluxes file assumed UTC starting time: ', &
     &                utc_start

        endif ! (end of first_call block)

! define frac_day - the fractional Julian date
! include offset for starting time in UTC (in hours)
        frac_day = start_day + time/secs_per_day + utc_start/24.0

! output info to debug file
        write(40,*) 'num_nodes = ', num_nodes
        write(40,*) 'num_files = ', num_files
        write(40,*) 'num_times = ', num_times
        write(40,*)
        write(40,*) 'start_day = ', start_day
        write(40,*) 'time      = ', time
        write(40,*) 'frac_day  = ', frac_day

! now we must retrieve the radiative fluxes, and copy from the filled
! real*4 arrays of reduced size to full-size real*8 arrays

! first shortwave
        call get_set (real(frac_day), 'shortwave_flux      ', &
     &                flux_set, time_exst, &
     &                temp_arr_4, temp_arr_2, temp_arr_3, &
     &                flux_times, num_times, ni, nj, &
     &                num_files, max_files, window, &
     &                flux_time_files)
        call copy_arr(temp_arr_4, ni, nj, shortwave_in, max_ni, max_nj)

! next longwave
        call get_set (real(frac_day), 'longwave_flux       ', &
     &                flux_set, time_exst, &
     &                temp_arr_4, temp_arr_2, temp_arr_3, &
     &                flux_times, num_times, ni, nj, &
     &                num_files, max_files, window, &
     &                flux_time_files)
        call copy_arr(temp_arr_4, ni, nj, longwave_in, max_ni, max_nj)
!       write(*,*) 'time_exst, window = ', time_exst, window

! if the desired time is outside of the times supplied by the dataset,
! or if the data interpolation window is greater than the maximum, then
! quit with an error message
        if ( (.not. time_exst) .or. (window .gt. max_window) ) then
          write(11,*)
          write(11,*) 'Radiative flux data: time not present in file!'
          stop
        endif

        write(40,*) 'rad fluxes obtained'

! now the input radiative fluxes must be put on a triangular grid

! use the weightings to interpolate from the input grid to the node
! points
        write(40,*) 'interpolating longwave'
        call interp_data (longwave_d, longwave_in, weight_rad_node, &
     &                    in_elem_to_out_node, elem_nodes_in, &
     &                    node_i, node_j, &
     &                    num_nodes_in, num_nodes, num_elems_in, &
     &                    max_ni, max_nj, max_nodes)

        write(40,*) 'interpolating shortwave'
        call interp_data (shortwave_d, shortwave_in, weight_rad_node, &
     &                    in_elem_to_out_node, elem_nodes_in, &
     &                    node_i, node_j, &
     &                    num_nodes_in, num_nodes, num_elems_in, &
     &                    max_ni, max_nj, max_nodes)

! get the albedo (store in temp_arr_8)
        write(40,*) 'calculating albedo'
        call get_albedo (temp_arr_8, num_nodes, max_nodes)

! reduce the downwards shortwave flux at the surface by the albedo
! (and check for possible negative values from interpolation)
        write(40,*) 'reducing shortwave'
        do i_node = 1, num_nodes
          shortwave_d(i_node) = max( (1.0d0 - temp_arr_8(i_node)) * &
     &                               shortwave_d(i_node), &
     &                               0.0d0 )
        enddo

! set first_call to false, so subsequent calls will know that they're
! not the first call
        first_call = .false.

        close(40)

      return
      end
!-----------------------------------------------------------------------
      subroutine copy_arr(array_1, ni_1, nj_1, array_2, ni_2, nj_2)
        implicit none
        integer ni_1, nj_1, ni_2, nj_2, i, j
        real*4 array_1(ni_1,nj_1)
        real*8 array_2(ni_2,nj_2)

        do j = 1, nj_1
          do i = 1, ni_1
            array_2(i,j) = array_1(i,j)
          enddo
        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine file_exst (file_name, exst, fail)
        implicit none
        character file_name*50
        logical exst, fail

        inquire(file=file_name, exist=exst)
        if (exst .and. fail) then
          write(11,*)
          write(11,*) file_name, ' already exists!'
         stop
        endif

      return
      end
!-----------------------------------------------------------------------
      subroutine get_bracket (time, input_times, i_time, bracket, &
     &                        num_times)
        implicit none
        integer num_times, i_time
        real*4 time, input_times(num_times)
        logical bracket

        i_time = 0
        bracket = .false.
10      continue
          i_time = i_time + 1
          bracket = ( input_times(i_time) .le. time .and. &
     &                time .le. input_times(i_time+1) )
        if (.not. bracket .and. i_time .lt. num_times-1) goto 10

      return
      end
!-----------------------------------------------------------------------
      subroutine interp_set(in_set, data, data_label, time, &
     &                      data_1, data_2, input_times, &
     &                      i_time, num_times, ni, nj, &
     &                      time_files, num_files, max_files)
        implicit none
        integer ni, nj, i, j, num_times, i_time, num_files, max_files
        real*4 data_1(ni,nj), data_2(ni,nj)
        real*4 data(ni,nj), time, input_times(num_times), ratio
        character in_set*50, data_label*20
        character time_files(num_times)*50

! read in the data at the first of the bracketing times
        call read_2d_arr (time_files(i_time), data_1, data_label, &
     &                    input_times(i_time), ni, nj)

! read in the data at the second of the bracketing times
        call read_2d_arr (time_files(i_time+1), data_2, data_label, &
     &                    input_times(i_time+1), ni, nj)

! now interpolate these to the desired time
        ratio = (time                  - input_times(i_time)) &
     &        / (input_times(i_time+1) - input_times(i_time))
        do j = 1, nj
          do i = 1, ni
            data(i,j) = data_1(i,j) &
     &                + (data_2(i,j) - data_1(i,j)) * ratio
          enddo
        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine get_dims(in_file, data_label, t, rank, dim_sizes)
        implicit none
        real*4 t
        integer ret, read_only, sfstart, sfend, sd_id, sds_id
        integer sds_index, sfn2index, sfendacc, sfselect
        integer max_rank, rank, data_type, n_attrs, sfginfo
        parameter (max_rank = 3)
        parameter (read_only = 1)
        integer dim_sizes(max_rank)
        character data_label*20, in_file*50
        character dat_time_label*33, sds_name*33

! open in_file in read only mode
        sd_id = sfstart(in_file, read_only)

! create the data-time label, which we'll use as the name
        call get_label (dat_time_label, data_label, t)

! find index for this data set
        if (t .lt. 0.0) then
          sds_index = sfn2index(sd_id, data_label)
        else
          sds_index = sfn2index(sd_id, dat_time_label)
        endif

! get info on dataset
        sds_id = sfselect(sd_id, sds_index)
        ret = sfginfo(sds_id, sds_name, rank, dim_sizes, &
     &                data_type, n_attrs)
        call checkret(ret)

! find the id for this index
        sds_id = sfselect(sd_id, sds_index)

! close access to the dataset
        ret = sfendacc(sds_id)
        call checkret(ret)

! close access to the file
        ret = sfend(sd_id)
        call checkret(ret)

      return
      end
!-----------------------------------------------------------------------
      subroutine list_nodes (node_i, node_j, node_num, &
     &                       n_nodes_in, ni, nj)
        implicit none
        integer ni, nj, n_nodes_in, i, j, i_node
        integer node_i(n_nodes_in), node_j(n_nodes_in)
        integer node_num(ni,nj)

        i_node = 0
        do j = 1, nj
          do i = 1, ni
            i_node = i_node + 1
            node_i(i_node) = i
            node_j(i_node) = j
            node_num(i,j) = i_node
          enddo
        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine list_elems (elem_nodes_in, node_num_in, &
     &                       ni, nj, n_elems_in)
        implicit none
        integer ni, nj, n_elems_in, i, j, i_elem
        integer node_num_in(ni,nj), elem_nodes_in(n_elems_in,3)

        i_elem = 0
        do j = 1, nj-1
          do i = 1, ni-1

! define the first element in this grid box
            i_elem = i_elem + 1
            elem_nodes_in(i_elem,1) = node_num_in(i,j)
            elem_nodes_in(i_elem,2) = node_num_in(i+1,j+1)
            elem_nodes_in(i_elem,3) = node_num_in(i,j+1)

! define the second element in this grid box
            i_elem = i_elem + 1
            elem_nodes_in(i_elem,1) = node_num_in(i,j)
            elem_nodes_in(i_elem,2) = node_num_in(i+1,j)
            elem_nodes_in(i_elem,3) = node_num_in(i+1,j+1)

          enddo
        enddo

      return
      end
!-----------------------------------------------------------------------

! note: this subroutine has been modified from the original version
! to account for full-sized data arrays (and still reduced size integer
! arrays)

      subroutine interp_data (data_out, data_in, weight, &
     &                        i_elem_ae_min, elem_nodes_in, &
     &                        node_i, node_j, &
     &                        n_nodes_in, n_nodes_out, n_elems_in, &
     &                        max_ni, max_nj, max_nodes_out)
        implicit none
        integer n_nodes_in, n_nodes_out, n_elems_in
        integer max_ni, max_nj, max_nodes_out
        integer i_elem_ae_min(n_nodes_out)
        integer elem_nodes_in(n_elems_in,3)
        integer node_i(n_nodes_in), node_j(n_nodes_in)
        real*8 data_in(max_ni,max_nj), data_out(max_nodes_out)
        real*8 weight(max_nodes_out,3)
        integer i_node, i_elem, i1, j1, i2, j2, i3, j3

! loop over the output nodes
        do i_node = 1, n_nodes_out

! get the locations of the nodes for the surrounding element on the
! input grid
          i_elem = i_elem_ae_min(i_node)
          i1 = node_i(elem_nodes_in(i_elem,1))
          j1 = node_j(elem_nodes_in(i_elem,1))
          i2 = node_i(elem_nodes_in(i_elem,2))
          j2 = node_j(elem_nodes_in(i_elem,2))
          i3 = node_i(elem_nodes_in(i_elem,3))
          j3 = node_j(elem_nodes_in(i_elem,3))

! the data on the output grid is simply the weighted data from the input
! grid
          data_out(i_node) = data_in(i1,j1) * weight(i_node,1) &
     &                     + data_in(i2,j2) * weight(i_node,2) &
     &                     + data_in(i3,j3) * weight(i_node,3)

        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine get_albedo (albedo, num_nodes, max_nodes)
        implicit none
        integer max_nodes, num_nodes, i_node
        real*8 albedo(max_nodes)

        do i_node = 1, num_nodes
          albedo(i_node) = 0.06
        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine read_2d_arr(in_file, data, data_label, t, ni, nj)
        implicit none
        integer ni, nj, edges(2)
        real*4 data(ni,nj), t
        integer ret, read_only, sfstart, sfend, sd_id, sds_id
        integer start(2), stride(2), sds_index, sfn2index
        integer sfrdata, sfendacc, sfselect
        parameter (read_only = 1)
        character data_label*20, in_file*50, dat_time_label*33

! open in_file in read only mode
        sd_id = sfstart(in_file, read_only)

! create the data-time label, which we'll use as the name
        call get_label (dat_time_label, data_label, t)

! find index for this data set
        if (t .lt. 0.0) then
          sds_index = sfn2index(sd_id, data_label)
        else
          sds_index = sfn2index(sd_id, dat_time_label)
        endif

! find the id for this index
        sds_id = sfselect(sd_id, sds_index)

! set up start, stride, and edges to read in the entire dataset
        start(1) = 0
        start(2) = 0
        stride(1) = 1
        stride(2) = 1
        edges(1) = ni
        edges(2) = nj

! read in the dataset
        ret = sfrdata(sds_id, start, stride, edges, data)
        call checkret(ret)

! close access to the dataset
        ret = sfendacc(sds_id)
        call checkret(ret)

! close access to the file
        ret = sfend(sd_id)
        call checkret(ret)

      return
      end
!-----------------------------------------------------------------------
      subroutine read_vec_int(in_file, data, data_label, t, ni)
        implicit none
        integer ni
        real*4 t
        integer data(ni)
        integer ret, read_only, sfstart, sfend, sd_id, sds_id
        integer sds_index, sfn2index, sfrdata, sfendacc, sfselect
        parameter (read_only = 1)
        character data_label*20, in_file*50, dat_time_label*33
        
! open in_file in read only mode
        sd_id = sfstart(in_file, read_only)

! create the data-time label, which we'll use as the name
        call get_label (dat_time_label, data_label, t)
        
! find index for this data set
        if (t .lt. 0.0) then
          sds_index = sfn2index(sd_id, data_label)
        else
          sds_index = sfn2index(sd_id, dat_time_label)
        endif

! find the id for this index
        sds_id = sfselect(sd_id, sds_index)

! read in the dataset
        ret = sfrdata(sds_id, 0, 1, ni, data)
        call checkret(ret)

! close access to the dataset
        ret = sfendacc(sds_id)
        call checkret(ret)

! close access to the file
        ret = sfend(sd_id)
        call checkret(ret)

      return
      end
!-----------------------------------------------------------------------
      subroutine read_scalar(in_file, data, data_label, t)
        implicit none
        real*4 data, t
        integer ret, read_only, sfstart, sfend, sd_id, sds_id
        integer sds_index, sfn2index, sfrdata, sfendacc, sfselect
        parameter (read_only = 1)
        character data_label*20, in_file*50, dat_time_label*33
        
! open in_file in read only mode
        sd_id = sfstart(in_file, read_only)

! create the data-time label, which we'll use as the name
        call get_label (dat_time_label, data_label, t)
        
! find index for this data set
        if (t .lt. 0.0) then
          sds_index = sfn2index(sd_id, data_label)
        else
          sds_index = sfn2index(sd_id, dat_time_label)
        endif

! find the id for this index
        sds_id = sfselect(sd_id, sds_index)

! read in the dataset
        ret = sfrdata(sds_id, 0, 1, 1, data)
        call checkret(ret)

! close access to the dataset
        ret = sfendacc(sds_id)
        call checkret(ret)

! close access to the file
        ret = sfend(sd_id)
        call checkret(ret)

      return
      end
!-----------------------------------------------------------------------
      subroutine get_label (dat_time_label, data_label, t)
        implicit none
        character data_label*20, dat_time_label*33, time_label*12
        character zero_short*12
        real*4 t
        integer i
        logical nonzero
        parameter (zero_short = '      0.    ')

! create the time_label
        write(time_label,10) t
10      format(g12.6)

! since different platforms use different formats for zero, we'll use a
! standard form
        nonzero = .false.
        do i = 1, 12
          if (time_label(i:i) .ge. '1' .and. &
     &        time_label(i:i) .le. '9') nonzero = .true.
        enddo
        
        if (.not. nonzero) time_label = zero_short

! create the data-time label
        write(dat_time_label,20) data_label, ' ', time_label
20      format(a20,a1,a12)

      return
      end
!-----------------------------------------------------------------------
      subroutine checkret (ret)
        implicit none
        integer ret
        
        if (ret .ne. 0) then
          write(11,*) 'nonzero HDF return code!'
          write(11,*) 'ret = ', ret
          write(11,*)
          stop
!       else
!         write(*,*) 'HDF file access ok. . .'
        endif
        
      return
      end
!-----------------------------------------------------------------------
      subroutine get_weight (x_in, y_in, x_out, y_out, &
     &                       elem_nodes_in, node_i, node_j, &
     &                       max_ni, max_nj, &
     &                       n_elems_in, n_nodes_in, &
     &                       n_nodes_out, &
     &                       max_nodes_out, &
     &                       i_elem_ae_min, &
     &                       area_in, weight)
        implicit none
        integer max_ni, max_nj, n_elems_in, n_nodes_in
        integer n_nodes_out, max_nodes_out
        integer node_i(n_nodes_in), node_j(n_nodes_in)
        integer elem_nodes_in(n_elems_in,3)
        real*8 x_in(max_ni,max_nj), y_in(max_ni,max_nj)
        real*8 x_out(n_nodes_out), y_out(n_nodes_out)
        real*8 area_in(n_elems_in)
        real*8 weight(max_nodes_out,3)
        integer i_elem, i_node, i_elem_ae_min(n_nodes_out)
        integer i1, j1, i2, j2, i3, j3
        integer last_elem, top, floor
        real*8 x1, y1, x2, y2, x3, y3, x4, y4, a1, a2, a3, aa, ae
        real*8 ae_min, epsilon
        parameter (epsilon = 1.0d-10)
        real*8 bad_point_flag
        parameter (bad_point_flag = -9.9d99)
        logical zero_ae, completed_check

! calculate and store the areas of the input grid elements
        do i_elem = 1, n_elems_in

          i1 = node_i(elem_nodes_in(i_elem,1))
          j1 = node_j(elem_nodes_in(i_elem,1))
          x1 = x_in(i1,j1)
          y1 = y_in(i1,j1)

          i2 = node_i(elem_nodes_in(i_elem,2))
          j2 = node_j(elem_nodes_in(i_elem,2))
          x2 = x_in(i2,j2)
          y2 = y_in(i2,j2)

          i3 = node_i(elem_nodes_in(i_elem,3))
          j3 = node_j(elem_nodes_in(i_elem,3))
          x3 = x_in(i3,j3)
          y3 = y_in(i3,j3)

          area_in(i_elem) = 0.5 * &
     &                      ( (x1-x3)*(y2-y3) + (x3-x2)*(y1-y3) )

        enddo

! now loop over the nodes of the output grid, searching for the
! surrounding elements on the input grid
!
! for first output node, begin search with first input element
        last_elem = 0
        do i_node = 1, n_nodes_out

          ae_min = 1.0e25
          i_elem_ae_min(i_node) = 0

! initialize flag which indicates that we've found correct element
! (where ae .eq. 0)
          zero_ae = .false.

! search for element, starting with location of previous node
100       continue

! this block implements logic which tells which is the next element
! to consider

! first output node, first input element
            if (last_elem .eq. 0) then
              i_elem = 1
              top = i_elem
              floor = i_elem

! first iteration with new output node
! (begin search at correct element for previous node)
            else if ( (top .eq. 0) .and. (floor .eq. 0) ) then
              i_elem = last_elem
              top = i_elem
              floor = i_elem

! if we searched the "floor" value last
            else if (last_elem .eq. floor) then
              if (top .eq. n_elems_in) then
                i_elem = floor - 1
                floor = floor - 1
              else
                i_elem = top + 1
                top = top + 1
              endif

! if we searched the "top" value last
            else if (last_elem .eq. top) then
              if (floor .eq. 1) then
                i_elem = top + 1
                top = top + 1
              else
                i_elem = floor - 1
                floor = floor - 1
              endif

! error in logic somehow (shouldn't happen, unless I screwed up code)
            else
              write(11,*)
              write(11,*) 'search error in get_weight!'
              stop
            endif

! get the locations of the nodes for this element on the input grid
            i1 = node_i(elem_nodes_in(i_elem,1))
            j1 = node_j(elem_nodes_in(i_elem,1))
            x1 = x_in(i1,j1)
            y1 = y_in(i1,j1)

            i2 = node_i(elem_nodes_in(i_elem,2))
            j2 = node_j(elem_nodes_in(i_elem,2))
            x2 = x_in(i2,j2)
            y2 = y_in(i2,j2)

            i3 = node_i(elem_nodes_in(i_elem,3))
            j3 = node_j(elem_nodes_in(i_elem,3))
            x3 = x_in(i3,j3)
            y3 = y_in(i3,j3)
!           write(*,*) i_elem, x1, x2, x3
!           write(*,*) i_elem, y1, y2, y3

! get the locations of this node for the output grid
            x4 = x_out(i_node)
            y4 = y_out(i_node)

! calculate some type of areas
            a1 = (x4-x3)*(y2-y3) + (x2-x3)*(y3-y4)
            a2 = (x4-x1)*(y3-y1) - (y4-y1)*(x3-x1)
            a3 = (y4-y1)*(x2-x1) - (x4-x1)*(y2-y1)
            aa = abs(a1) + abs(a2) + abs(a3)
            if (area_in(i_elem) .gt. 0.0) then
              ae = abs(aa - 2.0*area_in(i_elem)) &
     &           / (2.0*area_in(i_elem))
            else
              ae = 1.0e25
            endif

! if ae equals zero (within epsilon) then we've found correct element
            zero_ae = (ae .lt. epsilon)

! determine if this is the smallest ae
! (element of input grid that contains node for output grid)
            if (ae .lt. ae_min) then
              ae_min = ae
              i_elem_ae_min(i_node) = i_elem
            endif

! have we checked all of the elements?
            completed_check = (floor .eq. 1) .and. (top .eq. n_elems_in)

! change last_elem pointer
            last_elem = i_elem

! loop again if need to
          if ( (.not. completed_check) .and. (.not. zero_ae) ) goto 100

! if we didnt find a good ae_min, then there are problems
          if (i_elem_ae_min(i_node) .eq. 0) then
            write(11,*)
            write(11,*) 'Couldn''t find suitable element in input grid'
            write(11,*) 'for output node #', i_node
            write(11,*) 'x_out, y_out = ', x_out(i_node), y_out(i_node)
            write(11,*) 'i_elem_ae_min, ae_min = ', &
     &                   i_elem_ae_min(i_node), ae_min
            stop
          endif

! the proper element has been found, now calculate the averaging weights
! (but make sure that there are no bad points from get_xy)

! get the locations of the nodes for this element on the input grid
          i_elem = i_elem_ae_min(i_node)
          i1 = node_i(elem_nodes_in(i_elem,1))
          j1 = node_j(elem_nodes_in(i_elem,1))
          x1 = x_in(i1,j1)
          y1 = y_in(i1,j1)

          i2 = node_i(elem_nodes_in(i_elem,2))
          j2 = node_j(elem_nodes_in(i_elem,2))
          x2 = x_in(i2,j2)
          y2 = y_in(i2,j2)

          i3 = node_i(elem_nodes_in(i_elem,3))
          j3 = node_j(elem_nodes_in(i_elem,3))
          x3 = x_in(i3,j3)
          y3 = y_in(i3,j3)

! check to make sure none of locations has been flagged as bad
          if ( (x1 .le. bad_point_flag) .or. &
     &         (x2 .le. bad_point_flag) .or. &
     &         (x3 .le. bad_point_flag) ) then
            write(11,*)
            write(11,*) 'Bad x-y flag from input grid'
            write(11,*) 'for output node #', i_node
            write(11,*) 'x_out, y_out = ', x_out(i_node), y_out(i_node)
            write(11,*) 'i_elem_ae_min, ae_min = ', &
     &                   i_elem_ae_min(i_node), ae_min
            stop
          endif

! get the locations of this node for the output grid
          x4 = x_out(i_node)
          y4 = y_out(i_node)

! now calculate the weighting functions
          weight(i_node,1) = ( (x4-x3)*(y2-y3) + (x2-x3)*(y3-y4) ) &
     &                     / ( 2.0*area_in(i_elem) )
          weight(i_node,2) = ( (x4-x1)*(y3-y1) - (y4-y1)*(x3-x1) ) &
     &                     / ( 2.0*area_in(i_elem) )
          weight(i_node,3) = ( -(x4-x1)*(y2-y1) + (y4-y1)*(x2-x1) ) &
     &                     / ( 2.0*area_in(i_elem) )

!999       format(2i8,g14.5,4f8.4)
!         if (mod(i_node-1,1000) .eq. 0)                               &
!    &      write(50,999) i_node, i_elem_ae_min(i_node), ae_min,       &
!    &           weight(i_node,1), weight(i_node,2), weight(i_node,3), &
!    &           weight(i_node,1) + weight(i_node,2) + weight(i_node,3)

          top = 0
          floor = 0

        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine get_set (time, data_label, &
     &                    in_set, time_exst, &
     &                    data_out, temp_arr_1, temp_arr_2, &
     &                    input_times, num_times, ni, nj, &
     &                    num_files, max_files, window, &
     &                    time_files)
        implicit none
        integer ni, nj, num_times, num_files, max_files, time_num
        real*4 time, data_out(ni,nj), input_times(num_times)
        real*4 temp_arr_1(ni,nj), temp_arr_2(ni,nj), window
        character data_label*20, in_set*50
        character time_files(num_times)*50
        logical time_exst, exact_time_exst
        integer first_bracket
        logical bracket

! check to see if data exists at the desired time (and the file it's in)
        call does_exact_time_exst (time, input_times, num_times, &
     &                             time_num, exact_time_exst)

! if data exists at the exact time, read it in directly
        if (exact_time_exst) then
          call read_2d_arr(time_files(time_num), data_out, data_label, &
     &                     time, ni, nj)
          time_exst = .true.
          window = 0.0

! or, if data doesn't exist at this exact time
        else

! check to see if/when times exist which bracket the desired time
          call get_bracket (time, input_times, first_bracket, &
     &                      bracket, num_times)

! if bracketing times exist, then read in the data at the bracketing
! times and interpolate to the desired time
          if (bracket) then

            call interp_set(in_set, data_out, data_label, time, &
     &                      temp_arr_1, temp_arr_2, input_times, &
     &                      first_bracket, num_times, ni, nj, &
     &                      time_files, num_files, max_files)

! calculate the width of the bracketing window (convert to hours)
            window = ( input_times(first_bracket+1) - &
     &                 input_times(first_bracket) ) * 24.0

            time_exst = .true.
          else
            time_exst = .false.
          endif

        endif

      return
      end
!-----------------------------------------------------------------------
      subroutine combine_set (time, data_label, &
     &                        in_set_1, in_set_2, &
     &                        data_1, data_2, &
     &                        temp_arr_1, temp_arr_2, temp_arr_3, &
     &                        input_times_1, num_times_1, ni_1, nj_1, &
     &                        input_times_2, num_times_2, ni_2, nj_2, &
     &                        num_files_1, num_files_2, max_files, &
     &                        data_1_node, data_2_node, data_combo_node, &
     &                        in_elem_to_out_node_1, weight_node_1, &
     &                        in_elem_to_out_node_2, weight_node_2, &
     &                        elem_nodes_in_1, node_i_1, node_j_1, &
     &                        elem_nodes_in_2, node_i_2, node_j_2, &
     &                        max_ni, max_nj, max_nodes_in, &
     &                        num_nodes_out, max_nodes_out, &
     &                        num_nodes_in_1, num_elems_in_1, &
     &                        num_nodes_in_2, num_elems_in_2, &
     &                        max_elems_in, have_wind_2, &
     &                        relative_weight_1, relative_weight_2, &
     &                        max_window_1, max_window_2, &
     &                        time_files_1, time_files_2)
        implicit none
        integer max_ni, max_nj, max_elems_in, i_node
        integer num_nodes_out, max_nodes_out, max_nodes_in
        integer num_nodes_in_1, num_elems_in_1
        integer num_nodes_in_2, num_elems_in_2
        integer num_files_1, num_files_2, max_files
        integer ni_1, nj_1, num_times_1
        integer ni_2, nj_2, num_times_2
        real*4 input_times_1(num_times_1), input_times_2(num_times_2)
        real*4 data_1(ni_1, nj_1), data_2(ni_2, nj_2), time
        real*4 temp_arr_1(max_ni, max_nj), temp_arr_2(max_ni, max_nj)
        real*4 window_1, window_2
        real*8 max_window_1, max_window_2
        real*8 temp_arr_3(max_ni, max_nj)
        real*8 data_1_node(max_nodes_out), data_2_node(max_nodes_out)
        real*8 data_combo_node(max_nodes_out)
        real*8 weight_node_1(max_nodes_out,3)
        real*8 weight_node_2(max_nodes_out,3)
        real*8 relative_weight_1, relative_weight_2
        real*8 local_weight_2, sum_weights
        integer in_elem_to_out_node_1(max_nodes_out)
        integer in_elem_to_out_node_2(max_nodes_out)
        integer elem_nodes_in_1(max_elems_in,3)
        integer elem_nodes_in_2(max_elems_in,3)
        integer node_i_1(max_nodes_in), node_j_1(max_nodes_in)
        integer node_i_2(max_nodes_in), node_j_2(max_nodes_in)
        character data_label*20, in_set_1*50, in_set_2*50
        character time_files_1(num_times_1)*50
        character time_files_2(num_times_2)*50
        logical have_wind_2, time_exst_1, time_exst_2, bad_node_2
        logical use_wind_2

! wind_set_1
        call get_set (time, data_label, &
     &                in_set_1, time_exst_1, &
     &                data_1, temp_arr_1, temp_arr_2, &
     &                input_times_1, num_times_1, ni_1, nj_1, &
     &                num_files_1, max_files, window_1, &
     &                time_files_1)

! if we don't have data at this time for set 1, then bomb out with error
! message (or if window width is greater than maximum)
        if ( (.not. time_exst_1) .or. &
     &       (window_1 .gt. max_window_1) ) then
          write(11,*)
          write(11,*) 'Wind forcing data 1: time not present in files!'
          stop
        endif

! copy the data to a full size real*8 array (temp_arr_3)
        call copy_arr(data_1, ni_1, nj_1, temp_arr_3, max_ni, max_nj)

! use the weightings to interpolate from the input grid to the node
! points
        call interp_data (data_1_node, temp_arr_3, weight_node_1, &
     &                    in_elem_to_out_node_1, elem_nodes_in_1, &
     &                    node_i_1, node_j_1, &
     &                    num_nodes_in_1, num_nodes_out, num_elems_in_1, &
     &                    max_ni, max_nj, max_nodes_out)

! wind_set_2
        if (have_wind_2) then

          call get_set (time, data_label, &
     &                  in_set_2, time_exst_2, &
     &                  data_2, temp_arr_1, temp_arr_2, &
     &                  input_times_2, num_times_2, ni_2, nj_2, &
     &                  num_files_2, max_files, window_2, &
     &                  time_files_2)

        else
          time_exst_2 = .false.
        endif

! even if we have data for this time, only use it if the interpolation
! window isn't too large
        use_wind_2 = time_exst_2 .and. (window_2 .le. max_window_2)

! if we have usable data at this time for set 2, then copy to full size
! array and interpolate to node points (as above)
        if (use_wind_2) then
          call copy_arr(data_2, ni_2, nj_2, temp_arr_3, max_ni, max_nj)

! use the weightings to interpolate from the input grid to the node
! points
          call interp_data (data_2_node, temp_arr_3, weight_node_2, &
     &                    in_elem_to_out_node_2, elem_nodes_in_2, &
     &                    node_i_2, node_j_2, &
     &                    num_nodes_in_2, num_nodes_out, num_elems_in_2, &
     &                    max_ni, max_nj, max_nodes_out)

        endif

! loop over all the nodes, calculating the combined value
        do i_node = 1, num_nodes_out

! determine if this node is not enclosed within the grid of data set 2
          bad_node_2 = ( &
     &        weight_node_2(i_node,1) .lt. 0.0 .or. &
     &        weight_node_2(i_node,1) .gt. 1.0 .or. &
     &        weight_node_2(i_node,2) .lt. 0.0 .or. &
     &        weight_node_2(i_node,2) .gt. 1.0 .or. &
     &        weight_node_2(i_node,3) .lt. 0.0 .or. &
     &        weight_node_2(i_node,3) .gt. 1.0 &
     &                 )

! calculate a local weight for the second data set (depending on whether
! the data exists at this time and place)
          if (use_wind_2 .and. .not. bad_node_2) then
            local_weight_2 = relative_weight_2
          else
            local_weight_2 = 0.0
          endif

! calculate the sum of the weightings
          sum_weights = relative_weight_1 + local_weight_2

! calculate a combined data value
          data_combo_node(i_node) = &
     &      ( relative_weight_1 * data_1_node(i_node) + &
     &        local_weight_2    * data_2_node(i_node) ) / sum_weights

        enddo

      return
      end
!-----------------------------------------------------------------------
      subroutine get_times(times, set_name, seek_label, &
     &                     time_files, num_times, &
     &                     num_files, max_times, max_files)
        implicit none
        integer max_times, num_times, file_number, max_files, num_files
        real*4 times(max_times), times_tmp
        character set_name*50, in_file*50, set_number*3, suffix*3
        character sds_name*33, data_label*20, time_label*12
        character seek_label*20, last_label*12, time_files(max_times)*50
        character test_time_label*12, test_dat_time_label*33
        logical exst, repeat, at_end
        integer sd_id_in, sfstart, read_only, ret, sffinfo, rank, i_rank
        integer n_datasets, n_file_attrs, sds_index, sfginfo, max_rank
        integer data_type, n_attrs, sds_id, sfendacc, sfend, sfselect
        parameter (max_rank = 3)
        parameter (read_only = 1)
        integer dim_sizes(max_rank), dim_sizes_1(max_rank), i_time

        suffix = 'hdf'
        num_times = 0
        num_files = 0

        write(16,*)
        write(16,*) 'building database of times/files for: ', set_name
! loop over the possible file names
        file_number = 0
100     continue
          file_number = file_number + 1

110       format(a2,i1)
120       format(a1,i2)
130       format(i3)
          if (file_number .lt. 10) then
            write(set_number,110) '00', file_number
          else if (file_number .lt. 100) then
            write(set_number,120) '0', file_number
          else if (file_number .lt. 1000) then
            write(set_number,130) file_number
          else if (file_number .gt. max_files) then
            write(11,*)
            write(11,*) 'too many input files!'
            stop
          endif

! construct the file name
200       format(a15,a1,a3,a1,a3)
          write(in_file,200) set_name(1:15), '.', set_number, '.', &
     &                       suffix
!         write(*,*) in_file

! check to see if this file exists
          call file_exst (in_file, exst, .false.)
          if (exst) then

            num_files = num_files + 1
!           write(*,*) in_file,  ' exists'

! open file in read-only mode
            sd_id_in = sfstart(in_file, read_only)

! get info on input file
            ret = sffinfo(sd_id_in, n_datasets, n_file_attrs)
            call checkret(ret)

! step through the datasets
            do sds_index = 0, n_datasets-1

! get info on dataset
              sds_id = sfselect(sd_id_in, sds_index)
              ret = sfginfo(sds_id, sds_name, rank, dim_sizes, &
     &                      data_type, n_attrs)
              call checkret(ret)
!             write(*,*) sds_index, ': ', sds_name

! if this is the field we seek, then check the time label
              data_label = sds_name(1:20)
              if (data_label .eq. seek_label) then
                time_label = sds_name(22:33)
                read(time_label,*) times_tmp
!               write(*,*) 'time = ', times_tmp

! if this is the first time read, then store it and file name
                if (num_times .eq. 0) then

                  num_times = num_times + 1
                  times(num_times) = times_tmp
                  time_files(num_times) = in_file

! store the dimensions of this dataset, so we can compare with others
                  do i_rank = 1, rank
                    dim_sizes_1(i_rank) = dim_sizes(i_rank)
                  enddo

! if this is not the first time read, then we need to make sure it's
! not a repeat
                else

! first, check the dimensions to ensure that we haven't changed
! resolutions (if so, then bomb out)
                  do i_rank = 1, rank
                    if (dim_sizes(i_rank) .ne. dim_sizes_1(i_rank)) then
                      write(11,*)
                      write(11,*) 'resolution change in: ', in_file
                      stop
                    endif
                  enddo

! loop over previous times, comparing present time label with their
! time labels
                  repeat = .false.
                  i_time = 0
400               continue
                    i_time = i_time + 1

                    call get_label (test_dat_time_label, seek_label, &
     &                              times(i_time))

                    test_time_label = test_dat_time_label(22:33)

                    repeat = repeat .or. &
     &                       (test_time_label .eq. time_label)

                  if ( (i_time .lt. num_times) .and. (.not. repeat) ) &
     &            goto 400

!                 write(*,*)
!                 write(*,*) 'testing time = ', time_label
!                 write(*,*) 'label is a repeat: ', repeat

! if time is _not_ a repeat, then check to see if it would be at the
! end of the list of times
                  if (.not. repeat) then
                    at_end = (times_tmp .gt. times(num_times))
                  endif

! if the time label is _not_ a repeat, _and_ it would be at end of list,
! then add it and file name to the list
                  if ( (.not. repeat) .and. at_end ) then
                    num_times = num_times + 1
                    if (num_times .gt. max_times) then
                      write(11,*)
                      write(11,*) 'maximum number of times exceeded!'
                      stop
                    endif
                    times(num_times) = times_tmp
                    time_files(num_times) = in_file
!                   write(*,*) 'num_times, time: ',
!    +                         num_times, times(num_times)
                  else if (repeat) then
! if it is a repeat, then use this file for this time
                    time_files(i_time) = in_file
! endif for the not repeat block
                  endif

! endif for the first time block
                endif

! endif for the seek_label block
              endif

! terminate access to this dataset
              ret = sfendacc(sds_id)
              call checkret(ret)

            enddo

! close HDF input file
            ret = sfend(sd_id_in)
            call checkret(ret)

! endif for the file exst block
          endif

! end of loop over possible file names
        if (exst) goto 100

!     write(40,*)
!     write(40,*) 'data available at:'
!25    format ( 6(g12.6, 1x) )
!     write(40,25) (times(i_time), i_time = 1, num_times)
!     write(40,*)

      return
      end
!-----------------------------------------------------------------------
      subroutine does_exact_time_exst (time, times, num_times, &
     &                                 time_num, exact_time_exst)
        implicit none
        integer num_times, time_num, i
        real*4 times(num_times), time
        logical exact_time_exst, nonzero
        character time_label*12, test_label*12
        character zero_short*12
        parameter (zero_short = '      0.    ')

! create the time_label
        write(time_label,10) time
10      format(g12.6)

! use a standard form for a zero label
        nonzero = .false.
        do i = 1, 12
          if (time_label(i:i) .ge. '1' .and. &
     &        time_label(i:i) .le. '9') nonzero = .true.
        enddo
        if (.not. nonzero) time_label = zero_short

!       write(*,*)
!       write(*,*) 'seeking for time_label: ', time_label

! loop over input times, creating their labels, and comparing
        time_num = 0
        exact_time_exst = .false.
100     continue
          time_num = time_num + 1

! create the time_label
          write(test_label,10) times(time_num)

! once again, use a standard form for a zero label
          nonzero = .false.
          do i = 1, 12
            if (time_label(i:i) .ge. '1' .and. &
     &          time_label(i:i) .le. '9') nonzero = .true.
          enddo
          if (.not. nonzero) time_label = zero_short

! compare labels
          exact_time_exst = (time_label .eq. test_label)
!         write(*,*) 'test_label = ', test_label
!         write(*,*) 'exact_time_exst = ', exact_time_exst

! bottom of loop
          if ( (.not. exact_time_exst) .and. (time_num .lt. num_times) ) &
     &    goto 100
        
      return
      end
!-----------------------------------------------------------------------
      subroutine get_xy (in_file, coord_1_r4, coord_2_r4, &
     &                   coord_1_r8, coord_2_r8, &
     &                   coord_1_r8_tmp, coord_2_r8_tmp, &
     &                   point_status, ni, nj, max_ni, max_nj)
        implicit none
        integer ni, nj, max_ni, max_nj, i, j, ni_nj
        real*4 coord_1_r4(ni,nj), coord_2_r4(ni,nj)
        real*8 coord_1_r8_tmp(ni,nj), coord_2_r8_tmp(ni,nj)
        real*8 coord_1_r8(max_ni,max_nj), coord_2_r8(max_ni,max_nj)
        real*8 bad_point_flag, pi, deg_to_rad
        integer point_status(ni,nj), setup_status
        parameter (bad_point_flag = -9.99d99)
        character in_file*50

        pi = 4.0 * atan(1.0d0)
        deg_to_rad = pi / 180.0d0

! read in lon and lat values to real*4 filled arrays
        call read_2d_arr(in_file, coord_1_r4, &
     &                   'lon                 ', 0.0, &
     &                   ni, nj)
        call read_2d_arr(in_file, coord_2_r4, &
     &                   'lat                 ', 0.0, &
     &                   ni, nj)

! confine lon to -180->180 range
        do j = 1, nj
          do i = 1, ni
            if (coord_1_r4(i,j) .gt. 180) then
              coord_1_r4(i,j) = coord_1_r4(i,j) - 360.0d0
            endif
          enddo
        enddo

! copy the lon-lat values to the (unfilled) output arrays,
! and convert degrees to radians
        do j = 1, nj
          do i = 1, ni
            coord_1_r8(i,j) = coord_1_r4(i,j) * deg_to_rad
            coord_2_r8(i,j) = coord_2_r4(i,j) * deg_to_rad
          enddo
        enddo

      return
      end
!-----------------------------------------------------------------------
#endif          /* end of USE_HDF block */
